﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Numerics;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace UseUwpApiWPF
{
    /// <summary>
    /// MainWindow.xaml の相互作用ロジック
    /// </summary>
    public partial class MainWindow : Window
    {
        private Windows.UI.Input.Inking.InkStrokeContainer _inkStrokeContainer;

        public MainWindow()
        {
            InitializeComponent();
            this._inkStrokeContainer = new Windows.UI.Input.Inking.InkStrokeContainer();
            this.WPFInkCanvas.EditingMode = InkCanvasEditingMode.None;

            this.Shortcut123Button.Visibility = Visibility.Collapsed;
            this.Shortcut45Button.Visibility = Visibility.Collapsed;
            this.WorkingProgressBar.Visibility = Visibility.Collapsed;
        }

        private async void LoadToInkStrokeContainerButton_Click(object sender, RoutedEventArgs e)
        {
            this.MyProgressBar.IsIndeterminate = true;

            Microsoft.Win32.OpenFileDialog dlgOpen = new Microsoft.Win32.OpenFileDialog();
            dlgOpen.Filter = "GIF with embedded ISF(*gif,*.gifmss)|*.gif;*.gifmss";
            if ((bool)dlgOpen.ShowDialog())
            {
                Windows.Storage.StorageFile file = await Windows.Storage.StorageFile.GetFileFromPathAsync(dlgOpen.FileName);
                Windows.Storage.Streams.IRandomAccessStream stream = await file.OpenAsync(Windows.Storage.FileAccessMode.Read);
                using (var inputStream = stream.GetInputStreamAt(0))
                {
                    try
                    {
                        await this._inkStrokeContainer.LoadAsync(inputStream);
                    }
                    catch (Exception e2)
                    {
                        this._inkStrokeContainer.Clear();
                    }
                }
                stream.Dispose();
            }

            this.MyProgressBar.IsIndeterminate = false;
        }

        private void CopyToClipboardFromInkStrokeContainerButton_Click(object sender, RoutedEventArgs e)
        {
            this.MyProgressBar.IsIndeterminate = true;

            var strokes = this._inkStrokeContainer.GetStrokes();
            foreach (var stroke in strokes)
            {
                stroke.Selected = true; ;
            }
            this._inkStrokeContainer.CopySelectedToClipboard();

            this.MyProgressBar.IsIndeterminate = false;
        }

        private void PasteFromClipboardButton_Click(object sender, RoutedEventArgs e)
        {
            this.MyProgressBar.IsIndeterminate = true;

            WPFInkCanvas.Strokes.Clear();
            if (WPFInkCanvas.CanPaste())
            {
                WPFInkCanvas.Paste(new Point(0, 0));
                var stroke = this.WPFInkCanvas.Strokes[this.WPFInkCanvas.Strokes.Count - 1];
                this.PasteFromClipboardTextBox.Text = stroke.GetBezierStylusPoints()[0].X.ToString() + " "
                    + stroke.GetBezierStylusPoints()[0].Y.ToString() + " "
                    + stroke.DrawingAttributes.Width.ToString() + " "
                    + stroke.DrawingAttributes.Height.ToString()
                    + " last stroke's information";

                var sc = new StrokeCollection();
                sc.Add(this.WPFInkCanvas.Strokes[this.WPFInkCanvas.Strokes.Count - 1]);
                this.WPFInkCanvas.Select(sc);
            }


            this.MyProgressBar.IsIndeterminate = false;
        }

        private void SaveAsISFButton_Click(object sender, RoutedEventArgs e)
        {
            this.MyProgressBar.IsIndeterminate = true;

            StrokeCollection currentStrokes = this.WPFInkCanvas.Strokes;
            if (currentStrokes.Count > 0)
            {
                Microsoft.Win32.SaveFileDialog dlgSave = new Microsoft.Win32.SaveFileDialog();
                dlgSave.Filter = "ISF(*.isf,*.isfmss)|*.isf;*.isfmss";

                if ((bool)dlgSave.ShowDialog())
                {
                    using (System.IO.FileStream fs =
                        new System.IO.FileStream(dlgSave.FileName, System.IO.FileMode.Create))
                    {
                        this.WPFInkCanvas.Strokes.Save(fs);
                    }
                }
            }

            this.MyProgressBar.IsIndeterminate = false;
        }

        private void MarkerButton_Click(object sender, RoutedEventArgs e)
        {
            this.MyProgressBar.IsIndeterminate = true;

            double d = 10;
            double penSize = d*2;

            Windows.UI.Input.Inking.InkPoint pointOne;
            //Windows.UI.Input.Inking.InkPoint pointTwo;
            List<Windows.UI.Input.Inking.InkPoint> points;
            Windows.UI.Input.Inking.InkStroke stroke;
            Windows.UI.Input.Inking.InkStrokeBuilder builder = new Windows.UI.Input.Inking.InkStrokeBuilder();
            Windows.UI.Input.Inking.InkDrawingAttributes ida = new Windows.UI.Input.Inking.InkDrawingAttributes();

            pointOne = new Windows.UI.Input.Inking.InkPoint(new Windows.Foundation.Point(d, d), 0.5f);
            //pointTwo = new Windows.UI.Input.Inking.InkPoint(new Windows.Foundation.Point(d, 0), 0.5f);
            points = new List<Windows.UI.Input.Inking.InkPoint>();
            points.Add(pointOne);
            //points.Add(pointTwo);
            stroke = builder.CreateStrokeFromInkPoints(points, Matrix3x2.Identity);
            //ida = this.InkCanvas1.InkPresenter.CopyDefaultDrawingAttributes();
            ida.Size = new Windows.Foundation.Size(penSize, penSize);
            ida.PenTip = Windows.UI.Input.Inking.PenTipShape.Rectangle;
            stroke.DrawingAttributes = ida;
            this._inkStrokeContainer.AddStroke(stroke);

            this.MarkerTextBox.Text = stroke.GetInkPoints()[0].Position.X.ToString() + " "
                + stroke.GetInkPoints()[0].Position.Y.ToString() + " "
                + stroke.DrawingAttributes.Size.Width.ToString() + " "
                + stroke.DrawingAttributes.Size.Height.ToString()
                + " Marker's point and pensize";

            this.MyProgressBar.IsIndeterminate = false;
        }

        private void Shortcut123Button_Click(object sender, RoutedEventArgs e)
        {
            this.WorkingProgressBar.IsIndeterminate = true;

            this.LoadToInkStrokeContainerButton_Click(new object(), new RoutedEventArgs());
            this.MarkerButton_Click(new object(), new RoutedEventArgs());
            this.CopyToClipboardFromInkStrokeContainerButton_Click(new object(), new RoutedEventArgs());

            this.WorkingProgressBar.IsIndeterminate = false;
        }

        private void Shortcut45Button_Click(object sender, RoutedEventArgs e)
        {
            this.WorkingProgressBar.IsIndeterminate = true;

            this.PasteFromClipboardButton_Click(new object(), new RoutedEventArgs());
            this.SaveAsISFButton_Click(new object(), new RoutedEventArgs());

            this.WorkingProgressBar.IsIndeterminate = false;
        }

        private void SaveAsPngButton_Click(object sender, RoutedEventArgs e)
        {
            this.MyProgressBar.IsIndeterminate = true;

            Microsoft.Win32.SaveFileDialog fileDialog = new Microsoft.Win32.SaveFileDialog();

            //fileDialog.Filter = "Ink Serialization Format(.isf)|*.isf" +
            //    "|ビットマップファイル(.bmp)|*.bmp|PNGファイル(.png)|*.png|JPEGファイル(.jpg)|*.jpg";
            //fileDialog.DefaultExt = "isf";
            fileDialog.Filter = "PNG(.png)|*.png";

            Nullable<bool> result = fileDialog.ShowDialog();
            if (result == true)
            {
                string fileName = fileDialog.FileName;

                // 拡張子から保存フォーマットを決定する
                string ext = System.IO.Path.GetExtension(fileName).ToLower();
                if (ext == ".isf")
                {
                    // ISF形式で保存する
                    FileStream fs = new FileStream(fileName, FileMode.Create);
                    this.WPFInkCanvas.Strokes.Save(fs);
                    fs.Close();
                }
                else
                {
                    // 画像フォーマットを指定して保存する

                    // Width・Heightにはサイズが設定されていないのでActualWidth・ActualHeightを利用する
                    Rect rect = new Rect(0, 0, this.WPFInkCanvas.ActualWidth, this.WPFInkCanvas.ActualHeight);

                    // 描画先の作成
                    DrawingVisual drawingVisual = new DrawingVisual();
                    DrawingContext context = drawingVisual.RenderOpen();

                    // inkCanvasの背景色で塗りつぶし
                    context.DrawRectangle(this.WPFInkCanvas.Background, null, rect);
                    // inkCanvasのストロークを描画する
                    this.WPFInkCanvas.Strokes.Draw(context);

                    context.Close();

                    // drawingVisualをビットマップに変更する
                    RenderTargetBitmap rtb = new RenderTargetBitmap(
                        (int)rect.Width, (int)rect.Height,
                        96, 96,
                        PixelFormats.Default);
                    rtb.Render(drawingVisual);

                    // 拡張子によってエンコーダーを変更する
                    BitmapEncoder enc = null;
                    switch (ext)
                    {
                        case ".png":
                            // PNG
                            enc = new PngBitmapEncoder();
                            break;
                        case ".jpg":
                            // JPEG
                            enc = new JpegBitmapEncoder();
                            break;
                        default:
                            // Bitmap
                            enc = new BmpBitmapEncoder();
                            break;
                    }
                    enc.Frames.Add(BitmapFrame.Create(rtb));

                    // ファイルへ保存
                    FileStream fs = new FileStream(fileName, FileMode.Create);
                    enc.Save(fs);
                    fs.Close();
                }
            }

            this.MyProgressBar.IsIndeterminate = false;
        }

        private void AnalyzeMarkerButton_Click(object sender, RoutedEventArgs e)
        {
            this.MyProgressBar.IsIndeterminate = true;

            var strokeCollection = this.WPFInkCanvas.Strokes;
            if (strokeCollection.Count > 0)
            {
                var stroke = strokeCollection[strokeCollection.Count - 1];
                var points = stroke.GetBezierStylusPoints();
                double startx = points[0].X;
                double starty = points[0].Y;
                double penwidth = stroke.DrawingAttributes.Width;
                double penheight = stroke.DrawingAttributes.Height;

                this.WPFInkCanvas.Select(strokeCollection);
                this.WPFInkCanvas.CopySelection();

                WPFInkCanvas.Strokes.Clear();
                if (WPFInkCanvas.CanPaste())
                {
                    WPFInkCanvas.Paste(new Point(penwidth/2 - startx, penheight/2 - starty));
                }

                stroke = this.WPFInkCanvas.Strokes[this.WPFInkCanvas.Strokes.Count - 1];
                this.AnalyzeMarkerButtonTextBox.Text = stroke.GetBezierStylusPoints()[0].X.ToString() + " "
                    + stroke.GetBezierStylusPoints()[0].Y.ToString() + " "
                    + stroke.DrawingAttributes.Width.ToString() + " "
                    + stroke.DrawingAttributes.Height.ToString()
                    + " last stroke's information";

                var sc = new StrokeCollection();
                sc.Add(this.WPFInkCanvas.Strokes[this.WPFInkCanvas.Strokes.Count - 1]);
                this.WPFInkCanvas.Select(sc);
            }

            this.MyProgressBar.IsIndeterminate = false;
        }

        private void ConvertStrokesButton_Click(object sender, RoutedEventArgs e)
        {
            this.MyProgressBar.IsIndeterminate = true;

            WPFInkCanvas.Strokes.Clear();
            if (this._inkStrokeContainer.GetStrokes().Count > 0)
            {
                var uwp_strokes = this._inkStrokeContainer.GetStrokes();
                foreach(var s in uwp_strokes)
                {
                    var points = s.GetInkPoints();
                    var wpf_points = new StylusPointCollection();
                    foreach(var p in points)
                    {
                        var wpf_point = new StylusPoint(p.Position.X, p.Position.Y, p.Pressure);
                        wpf_points.Add(wpf_point);
                    }
                    var wpf_da = new DrawingAttributes();
                    wpf_da.Color = Color.FromArgb(s.DrawingAttributes.Color.A,
                        s.DrawingAttributes.Color.R,
                        s.DrawingAttributes.Color.G,
                        s.DrawingAttributes.Color.B);
                    wpf_da.Width = s.DrawingAttributes.Size.Width;
                    wpf_da.Height = s.DrawingAttributes.Size.Height;
                    if(s.DrawingAttributes.PenTip == Windows.UI.Input.Inking.PenTipShape.Circle)
                    {
                        wpf_da.StylusTip = StylusTip.Ellipse;
                    } else if(s.DrawingAttributes.PenTip == Windows.UI.Input.Inking.PenTipShape.Rectangle)
                    {
                        wpf_da.StylusTip = StylusTip.Rectangle;
                    }
                    wpf_da.FitToCurve = s.DrawingAttributes.FitToCurve;
                    wpf_da.StylusTipTransform = new  Matrix(s.DrawingAttributes.PenTipTransform.M11,
                        s.DrawingAttributes.PenTipTransform.M12,
                        s.DrawingAttributes.PenTipTransform.M21,
                        s.DrawingAttributes.PenTipTransform.M22,
                        s.DrawingAttributes.PenTipTransform.M31,
                        s.DrawingAttributes.PenTipTransform.M32);
                    var wpf_stroke = new Stroke(wpf_points, wpf_da);
                    this.WPFInkCanvas.Strokes.Add(wpf_stroke);
                }

                var stroke = this.WPFInkCanvas.Strokes[this.WPFInkCanvas.Strokes.Count - 1];
                this.PasteFromClipboardTextBox.Text = stroke.GetBezierStylusPoints()[0].X.ToString() + " "
                    + stroke.GetBezierStylusPoints()[0].Y.ToString() + " "
                    + stroke.DrawingAttributes.Width.ToString() + " "
                    + stroke.DrawingAttributes.Height.ToString()
                    + " last stroke's information";

                var sc = new StrokeCollection();
                sc.Add(this.WPFInkCanvas.Strokes[this.WPFInkCanvas.Strokes.Count - 1]);
                this.WPFInkCanvas.Select(sc);
            }


            this.MyProgressBar.IsIndeterminate = false;
        }

        private void CopyToClipboardFromWPFInkCanvas_Click(object sender, RoutedEventArgs e)
        {
            this.MyProgressBar.IsIndeterminate = true;

            var strokeCollection = this.WPFInkCanvas.Strokes;
            if (strokeCollection.Count > 0)
            {
                InkCanvasEditingMode icem = this.WPFInkCanvas.EditingMode;

                this.WPFInkCanvas.Select(strokeCollection);
                this.WPFInkCanvas.CopySelection();

                //this.InkCanvas1.EditingMode = icem;
            }


            this.MyProgressBar.IsIndeterminate = false;
        }
    }
}
